@@ -153,6 +153,17 @@ cholmod_triplet attribute_hidden
     return fun(A, Common);
 }
 
+cholmod_dense attribute_hidden
+*M_cholmod_sparse_to_dense(cholmod_sparse *A, cholmod_common *Common)
+{
+    static cholmod_dense*(*fun)
+	(cholmod_sparse*,cholmod_common*) = NULL;
+    if (fun == NULL)
+	fun = (cholmod_dense*(*)(cholmod_sparse*,cholmod_common*))
+	    R_GetCCallable("Matrix", "cholmod_sparse_to_dense");
+    return fun(A, Common);
+}
+
 cholmod_factor attribute_hidden
 *M_cholmod_analyze(cholmod_sparse *A, cholmod_common *Common)
 {
@@ -237,6 +248,16 @@ M_cholmod_finish(cholmod_common *Common)
     return fun(Common);
 }
 
+int attribute_hidden
+M_cholmod_sort(cholmod_sparse *A, cholmod_common *Common)
+{
+    static int(*fun)(cholmod_sparse*,cholmod_common*) = NULL;
+    if (fun == NULL)
+	fun = (int(*)(cholmod_sparse*,cholmod_common*))
+	    R_GetCCallable("Matrix", "cholmod_sort");
+    return fun(A, Common);
+}
+
 int attribute_hidden
 M_cholmod_free_dense(cholmod_dense **A, cholmod_common *Common)
 {
@@ -339,14 +360,24 @@ cholmod_sparse attribute_hidden
     return fun(sys, L, B, Common);
 }
 
-int attribute_hidden
-M_cholmod_start(cholmod_common *Common)
+void attribute_hidden
+M_R_cholmod_error(int status, char *file, int line, char *message)
 {
+        error("Cholmod error `%s' at file:%s, line %d", message, file, line);
+}
+    
+int attribute_hidden
+M_R_cholmod_start(cholmod_common *Common)
+{
+    int val;
     static int(*fun)(cholmod_common*) = NULL;
     if (fun == NULL)
 	fun = (int(*)(cholmod_common*))
 	    R_GetCCallable("Matrix", "cholmod_start");
-    return fun(Common);
+    val = fun(Common);
+    Common->print_function = Rprintf;
+    Common->error_handler = M_R_cholmod_error;
+    return val;
 }
 
 cholmod_sparse attribute_hidden
